@@ -20,13 +20,17 @@ Authors:
 # Imports
 #-----------------------------------------------------------------------------
 
-from zmq       import green
-from zmq.utils import jsonapi
+from logging import getLogger
+
+from zmq import green
 
 from gevent       import spawn
 from gevent.event import Event, AsyncResult
 
-from ..client import RPCClientBase, RPCError, RemoteRPCError
+from ..client import RPCClientBase
+
+
+logger = getLogger("netcall")
 
 #-----------------------------------------------------------------------------
 # RPC Service Proxy
@@ -37,7 +41,7 @@ class GeventRPCClient(RPCClientBase):
         Uses the Gevent compatibility layer of pyzmq (zmq.green).
     """
 
-    def __init__(self, context=None, **kwargs):
+    def __init__(self, context=None, **kwargs):  #{
         """
         Parameters
         ==========
@@ -55,26 +59,26 @@ class GeventRPCClient(RPCClientBase):
         self.greenlet  = spawn(self._reader)
         self._results  = {}    # {<msg-id> : <gevent.AsyncResult>}
         super(GeventRPCClient, self).__init__(**kwargs)  # base class
-
-    def _create_socket(self):
+    #}
+    def _create_socket(self):  #{
         super(GeventRPCClient, self)._create_socket()
-
-    def bind(self, url):
+    #}
+    def bind(self, url):  #{
         result = super(GeventRPCClient, self).bind(url)
         self._ready_ev.set()  # wake up _reader
         return result
-
-    def bind_ports(self, *args, **kwargs):
+    #}
+    def bind_ports(self, *args, **kwargs):  #{
         result = super(GeventRPCClient, self).bind_ports(*args, **kwargs)
         self._ready_ev.set()  # wake up _reader
         return result
-
-    def connect(self, url):
+    #}
+    def connect(self, url):  #{
         result = super(GeventRPCClient, self).connect(url)
         self._ready_ev.set()  # wake up _reader
         return result
-
-    def _reader(self):
+    #}
+    def _reader(self):  #{
         """ Reader greenlet
 
             Waits for a socket to become ready (._ready_ev), then reads incoming replies and
@@ -97,31 +101,41 @@ class GeventRPCClient(RPCClientBase):
                     msg_list = socket.recv_multipart()
                 except Exception, e:
                     # the socket must have been closed
-                    print e
+                    logger.warning(e)
                     break
 
-                if not msg_list[0] == b'|':
-                    err_msg = 'Unexpected reply message format in GeventRPCClient._reader'
-                    print err_msg
-                    raise RPCError(err_msg)
+                logger.debug('received: %r' % msg_list)
 
-                # look for matching async result
-                msg_id = msg_list[1]
-                if msg_id in results:
-                    results[msg_id].set(msg_list)
-                    del results[msg_id]
+                reply = self._parse_reply(msg_list)
 
-        print 'EXIT'
+                if reply is None:
+                    continue
 
-    def shutdown(self):
+                req_id   = reply['req_id']
+                msg_type = reply['type']
+                result   = reply['result']
+
+                if msg_type == b'ACK':
+                    continue
+
+                async = results.pop(req_id, None)
+
+                if msg_type == b'OK':
+                    async.set(result)
+                else:
+                    async.set_exception(result)
+
+        logger.warning('_reader exited')
+    #}
+    def shutdown(self):  #{
         """Close the socket and signal the reader greenlet to exit"""
         self._ready = False
         self._exit_ev.set()
         self._ready_ev.set()
         self.socket.close()
         self._ready_ev.clear()
-
-    def call(self, method, *args, **kwargs):
+    #}
+    def call(self, method, args=[], kwargs={}, ignore=False):  #{
         """Call the remote method with *args and **kwargs.
 
         Parameters
@@ -142,19 +156,14 @@ class GeventRPCClient(RPCClientBase):
         if not self._ready:
             raise RuntimeError('bind or connect must be called first')
 
-        msg_id, msg_list = self._build_request(method, args, kwargs)
-        result = AsyncResult()
-        self._results[msg_id] = result
+        msg_id, msg_list = self._build_request(method, args, kwargs, ignore)
 
         self.socket.send_multipart(msg_list)
-        msg_list = result.get()  # block waiting for a reply passed by ._reader
-
-        status = msg_list[2]
-
-        if status == b'SUCCESS':
-            result = self._serializer.deserialize_result(msg_list[3:])
-            return result
-        elif status == b'FAILURE':
-            error_dict = jsonapi.loads(msg_list[3])
-            raise RemoteRPCError(error_dict['ename'], error_dict['evalue'], error_dict['traceback'])
 
+        if ignore:
+            return None
+        else:
+            result = AsyncResult()
+            self._results[msg_id] = result
+            return result.get()  # block waiting for a reply passed by ._reader
+    #}
